from django.core.management.base import NoArgsCommand
from django.conf import settings
import os
import sys
import re
import codecs
import commands
import urllib2
import urllib
import json
import numpy
import random
from time import sleep
import math
from django_bitcoin.models import RefillPaymentQueue, UpdatePayments

class Command(NoArgsCommand):
    help = 'Create a profile object for users which do not have one.'

    def handle_noargs(self, **options):
        RefillPaymentQueue()
        UpdatePayments()



# vim: tabstop=4 expandtab autoindent shiftwidth=4 fileencoding=utf-8

import os
import json
import jsonrpc
import sys
import urllib
import urllib2
import random
import hashlib
import base64
from decimal import Decimal
import decimal
import warnings

from django.core.cache import cache
from django.db import transaction

from django_bitcoin import settings
from django_bitcoin import currency

# BITCOIND COMMANDS

def quantitize_bitcoin(d):
    return d.quantize(Decimal("0.00000001"))

class BitcoindConnection(object):
    def __init__(self, connection_string, main_account_name):
        self.bitcoind_api = jsonrpc.ServiceProxy(connection_string)
        self.account_name = main_account_name

    def total_received(self, address, minconf=settings.BITCOIN_MINIMUM_CONFIRMATIONS):
        if settings.BITCOIN_TRANSACTION_CACHING:
            cache_key=address+"_"+str(minconf)
            cached = cache.get(cache_key)
            if cached!=None:
                return cached
            cached=decimal.Decimal(
                self.bitcoind_api.getreceivedbyaddress(address, minconf))
            cache.set(cache_key, cached, 5)
            return cached
        return decimal.Decimal(
                self.bitcoind_api.getreceivedbyaddress(address, minconf))
    
    def send(self, address, amount, *args, **kwargs):
        return self.bitcoind_api.sendtoaddress(address, float(amount), *args, **kwargs)

    def create_address(self, for_account=None, *args, **kwargs):
        return self.bitcoind_api.getnewaddress(
            for_account or self.account_name, *args, **kwargs)
    
    def gettransaction(self, txid, *args, **kwargs):
        dir (self.bitcoind_api)
        return self.bitcoind_api.gettransaction(txid, *args, **kwargs)

bitcoind = BitcoindConnection(settings.BITCOIND_CONNECTION_STRING,
                              settings.MAIN_ACCOUNT)

def bitcoin_getnewaddress(account_name=None):
    warnings.warn("Use bitcoind.create_address(...) instead",    
                  DeprecationWarning)
    return bitcoind.create_address(account_name=account_name)

def bitcoin_getbalance(address, minconf=1):
    warnings.warn("Use bitcoind.total_received(...) instead",
                  DeprecationWarning)
    return bitcoind.total_received(address, minconf)

def bitcoin_getreceived(address, minconf=settings.BITCOIN_MINIMUM_CONFIRMATIONS):
    warnings.warn("Use bitcoind.total_received(...) instead",
                  DeprecationWarning)
    return bitcoind.total_received(address, minconf)

def bitcoin_sendtoaddress(address, amount):
    warnings.warn("Use bitcoind.send(...) instead",
                  DeprecationWarning)
    return bitcoind.send(address, amount)

# --------

def bitcoinprice_usd():
    """return bitcoin price from any service we can get it"""
    warnings.warn("Use django_bitcoin.currency.exchange.get_rate('USD')",
                  DeprecationWarning)
    return {"24h": currency.exchange.get_rate("USD")}

def bitcoinprice_eur():
    warnings.warn("Use django_bitcoin.currency.exchange.get_rate('EUR')",
                  DeprecationWarning)
    return {"24h": currency.exchange.get_rate("EUR")}

def bitcoinprice(currency):
    warnings.warn("Use django_bitcoin.currency.exchange.get_rate(currency)",
                  DeprecationWarning)
    return currency.exchange.get_rate(currency)

# ------

# generate a random hash
def generateuniquehash(length=43, extradata=''):
    # cryptographically safe random
    r=str(os.urandom(64))
    m = hashlib.sha256()
    m.update(r+str(extradata))
    key=m.digest()
    key=base64.urlsafe_b64encode(key)
    return key[:min(length, 43)]

import string

ALPHABET = string.ascii_uppercase + string.ascii_lowercase + \
           string.digits + '_-'
ALPHABET_REVERSE = dict((c, i) for (i, c) in enumerate(ALPHABET))
BASE = len(ALPHABET)
SIGN_CHARACTER = '%'

def int2base64(n):
    if n < 0:
        return SIGN_CHARACTER + num_encode(-n)
    s = []
    while True:
        n, r = divmod(n, BASE)
        s.append(ALPHABET[r])
        if n == 0: break
    return ''.join(reversed(s))

def base642int(s):
    if s[0] == SIGN_CHARACTER:
        return -num_decode(s[1:])
    n = 0
    for c in s:
        n = n * BASE + ALPHABET_REVERSE[c]
    return n



# Create your views here.
from django.http import HttpResponseRedirect, HttpResponse
from django.core.cache import cache
import qrcode
import StringIO

def qrcode_view(request, key):
    cache_key="qrcode:"+key
    c=cache.get(cache_key)
    if not c:
        img = qrcode.make(key, box_size=4)
        output = StringIO.StringIO()
        img.save(output, "PNG")
        c = output.getvalue()
        cache.set(cache_key, c, 60*60)
    return HttpResponse(c, mimetype="image/png")


from django.core.management.base import NoArgsCommand
from django.conf import settings
import os
import sys
import re
import codecs
import commands
import urllib2
import urllib
import json
import random
from time import sleep
import math
from django_bitcoin.models import Wallet, BitcoinAddress, WalletTransaction
from django_bitcoin.utils import bitcoind

class Command(NoArgsCommand):
    help = 'This checks that alles is in ordnung in django_bitcoin.'

    def handle_noargs(self, **options):
        # BitcoinAddress.objects.filter(active=True)
        bitcoinaddress_sum = sum([x.received() for x in BitcoinAddress.objects.filter(active=True)])
        print "Total received, sum", bitcoinaddress_sum
        transaction_wallets_sum = sum([x.amount for x in WalletTransaction.objects.filter(from_wallet__id__gt=0, to_wallet__id__gt=0)])
        print "Total transactions, sum", transaction_wallets_sum
        transaction_out_sum = sum([x.amount for x in WalletTransaction.objects.filter(from_wallet__id__gt=0)\
        	.exclude(to_bitcoinaddress="").exclude(to_bitcoinaddress="")])
        print "Total outgoing, sum", transaction_out_sum
        # for x in WalletTransaction.objects.filter(from_wallet__id__gt=0, to_wallet__isnull=True, to_bitcoinaddress=""):
        # 	print x.amount, x.created_at
        fee_sum = sum([x.amount for x in WalletTransaction.objects.filter(from_wallet__id__gt=0, to_wallet__isnull=True, to_bitcoinaddress="")])
        print "Fees, sum", fee_sum
        print "----"
        bitcoind_balance = bitcoind.bitcoind_api.getbalance()
        print "Bitcoind balance", bitcoind_balance




#
# Django field type for a Bitcoin Address
#
import re
from django import forms
from django.forms.util import ValidationError
import hashlib

class BCAddressField(forms.CharField):
    default_error_messages = {
        'invalid': 'Invalid Bitcoin address.',
        }

    def __init__(self, *args, **kwargs):
        super(BCAddressField, self).__init__(*args, **kwargs)

    def clean(self, value):
        value = value.strip()
        if re.match(r"[a-zA-Z1-9]{27,35}$", value) is None:
            raise ValidationError(self.error_messages['invalid'])
        version = get_bcaddress_version(value)
        if version is None:
            raise ValidationError(self.error_messages['invalid'])
        return value

def is_valid_btc_address(value):
    value = value.strip()
    if re.match(r"[a-zA-Z1-9]{27,35}$", value) is None:
        return False
    version = get_bcaddress_version(value)
    if version is None:
        return False
    return True

import math

__b58chars = '123456789ABCDEFGHJKLMNPQRSTUVWXYZabcdefghijkmnopqrstuvwxyz'
__b58base = len(__b58chars)

def b58encode(v):
    """ encode v, which is a string of bytes, to base58.                                                                                                                                                                                                                             
    """

    long_value = 0L
    for (i, c) in enumerate(v[::-1]):
        long_value += (256**i) * ord(c)

    result = ''
    while long_value >= __b58base:
        div, mod = divmod(long_value, __b58base)
        result = __b58chars[mod] + result
        long_value = div
    result = __b58chars[long_value] + result

    # Bitcoin does a little leading-zero-compression:                                                                                                                                                                                                                                    
    # leading 0-bytes in the input become leading-1s                                                                                                                                                                                                                                     
    nPad = 0
    for c in v:
        if c == '\0': nPad += 1
        else: break

    return (__b58chars[0]*nPad) + result

def b58decode(v, length):
    """ decode v into a string of len bytes                                                                                                                                                                                                                                                        
    """
    long_value = 0L
    for (i, c) in enumerate(v[::-1]):
        long_value += __b58chars.find(c) * (__b58base**i)

    result = ''
    while long_value >= 256:
        div, mod = divmod(long_value, 256)
        result = chr(mod) + result
        long_value = div
    result = chr(long_value) + result

    nPad = 0
    for c in v:
        if c == __b58chars[0]: nPad += 1
        else: break

    result = chr(0)*nPad + result
    if length is not None and len(result) != length:
        return None

    return result

def get_bcaddress_version(strAddress):
    """ Returns None if strAddress is invalid.    Otherwise returns integer version of address. """
    addr = b58decode(strAddress,25)
    if addr is None: return None
    version = addr[0]
    checksum = addr[-4:]
    vh160 = addr[:-4] # Version plus hash160 is what is checksummed                                                                                                                                                                                                        
    h3=hashlib.sha256(hashlib.sha256(vh160).digest()).digest()
    if h3[0:4] == checksum:
        return ord(version)
    return None


# coding=utf8
# vim: ai ts=4 sts=4 et sw=4

from django.db import models
from django import forms
from django.conf import settings
from django.contrib.auth.models import User, AnonymousUser
from django.utils.translation import get_language_from_request, ugettext_lazy as _
from djangoextras.forms import CurrencyField
from django_bitcoin.models import BitcoinEscrow

class BitcoinEscrowBuyForm(ModelForm):
    class Meta:
        model=BitcoinEscrow
        fields = ('buyer_address', 'buyer_phone', 'buyer_email')
        



from django.conf import settings

MAIN_ACCOUNT = getattr(
    settings, 
    "BITCOIND_MAIN_ACCOUNT", 
    "somerandomstring14aqqwd")
BITCOIND_CONNECTION_STRING = getattr(
    settings, 
    "BITCOIND_CONNECTION_STRING", 
    "")
BITCOIN_PAYMENT_BUFFER_SIZE = getattr(
    settings, 
    "BITCOIN_PAYMENT_BUFFER_SIZE",
    5)
BITCOIN_ADDRESS_BUFFER_SIZE = getattr(
    settings, 
    "BITCOIN_ADDRESS_BUFFER_SIZE", 
    5)
PAYMENT_VALID_HOURS = getattr(
    settings, 
    "BITCOIND_PAYMENT_VALID_HOURS", 
    128)
REUSE_ADDRESSES = getattr(
    settings, 
    "BITCOIND_REUSE_ADDRESSES", 
    True)
ESCROW_PAYMENT_TIME_HOURS = getattr(
    settings, 
    "BITCOIND_ESCROW_PAYMENT_TIME_HOURS", 
    4)
ESCROW_RELEASE_TIME_DAYS = getattr(
    settings, 
    "BITCOIND_ESCROW_RELEASE_TIME_DAYS", 
    14)
BITCOIN_MINIMUM_CONFIRMATIONS = getattr(
    settings, 
    "BITCOIN_MINIMUM_CONFIRMATIONS", 
    3)
BITCOIN_TRANSACTION_CACHING = getattr(
    settings, 
    "BITCOIN_TRANSACTION_CACHING", 
    False)

BITCOIN_CURRENCIES = getattr(
    settings, 
    "BITCOIN_CURRENCIES",
    [
        "django_bitcoin.currency.BTCCurrency",
        "django_bitcoin.currency.EURCurrency",
        "django_bitcoin.currency.USDCurrency"
        ])



from django_bitcoin.models import bitcoinprice_eur, bitcoinprice_usd

def bitcoinprice(request):
    return {'bitcoinprice_eur': bitcoinprice_eur(),
        'bitcoinprice_usd': bitcoinprice_usd(),
        }



from django_bitcoin.models import Payment, new_bitcoin_payment
from django_bitcoin.models import Wallet, BitcoinAddress
from django_bitcoin.utils import generateuniquehash, int2base64, base642int, bitcoinprice, bitcoinprice_usd




# vim: tabstop=4 expandtab autoindent shiftwidth=4 fileencoding=utf-8

from django.contrib import admin

from django_bitcoin import models

class TransactionAdmin(admin.ModelAdmin):
    """Management of ``Transaction`` - Disable address etc editing
    """

    list_display = ('address', 'created_at', 'amount')
    readonly_fields = ('address', 'created_at', 'amount')


class BitcoinAddressAdmin(admin.ModelAdmin):
    """Deal with ``BitcoinAddress``
    No idea in being able to edit the address, as that would not
    sync with the network
    """

    list_display = ('address', 'label', 'created_at', 'least_received', 'active')
    readonly_fields = ('address',)


class PaymentAdmin(admin.ModelAdmin):
    """Allow the edit of ``description``
    """

    list_display = ('created_at', 'description', 'paid_at', 'address', 'amount', 'amount_paid', 'active')
    readonly_fields = ('address', 'amount', 'amount_paid', 'created_at', 'updated_at', 'paid_at', 'withdrawn_total', 'transactions')


class WalletTransactionAdmin(admin.ModelAdmin):
    """Inter-site transactions
    """

    list_display = ('created_at', 'from_wallet', 'to_wallet', 'to_bitcoinaddress', 'amount')
    readonly_fields = ('created_at', 'from_wallet', 'to_wallet', 'to_bitcoinaddress', 'amount')


class WalletAdmin(admin.ModelAdmin):
    """Admin ``Wallet``
    """

    list_display = ('created_at', 'label', 'updated_at')
    readonly_fields = ('created_at', 'updated_at', 'addresses', 'transactions_with')


admin.site.register(models.Transaction, TransactionAdmin)
admin.site.register(models.BitcoinAddress, BitcoinAddressAdmin)
admin.site.register(models.Payment, PaymentAdmin)
admin.site.register(models.WalletTransaction, WalletTransactionAdmin)
admin.site.register(models.Wallet, WalletAdmin)

# EOF




# -*- coding: utf-8 -*-
"""Usage:

>>> currency.exchange(
...     currency.Money("10.0", "BTC"), 
...     "BTC") 
Money("10.0", "BTC")

Default valid currencies are BTC, EUR and USD. Change exchange rate
sources and add new ones by the setting BITCOIN_CURRENCIES, which
should be a list of dotted paths to Currency subclasses (or other
classes) which implement both to_btc(decimal amount) -> decimal and
from_btc(decimal amount) -> decimal.

You can subclass or instance the `Exchange` class to e.g. maintain
multiple different exchange rates from different sources in your own
code. Default `exchange` uses Bitcoincharts.
"""

import decimal

from django.core.cache import cache

import json
import jsonrpc
import sys
import urllib
import urllib2
import random
import hashlib
import base64
from decimal import Decimal
import decimal
import warnings


from django_bitcoin import settings

class ConversionError(Exception):
    pass

class TemporaryConversionError(ConversionError):
    pass 

class Exchange(object):
    def __init__(self):
        self.currencies = {}

    def register_currency(self, klass):
        self.currencies[klass.identifier] = klass

    def get_rate(self, currency, target="BTC"):
        """Rate is inferred from a dummy exchange"""
        start = Money(currency, "1.0")
        end = self(start, target)
        return end.amount

    def __call__(self, money, target="BTC"):
        """Gets the current equivalent amount of the given Money in
        the target currency
        """
        if not hasattr(money, "identifier"):
            raise ConversionError(
                "Use annotated currency (e.g. Money) as "
                "the unit argument")

        if money.identifier not in self.currencies:
            raise ConversionError(
                "Unknown source currency %(identifier)s. "
                "Available currencies: %(currency_list)s" % {
                    "identifier": money.identifier, 
                    "currency_list": u", ".join(self.currencies.keys())})

        if target not in self.currencies:
            raise ConversionError(
                "Unknown target currency %(identifier)s. "
                "Available currencies: %(currency_list)s" % {
                    "identifier": target, 
                    "currency_list": u", ".join(self.currencies.keys())})

        btc = self.currencies[money.identifier].to_btc(money.amount)
        return Money(target, self.currencies[target].from_btc(btc))

class Money(object):
    def __init__(self, identifier, amount, *args, **kwargs):
        self.identifier = identifier
        self.amount = decimal.Decimal(amount)

    def __add__(self, other):
        if (not hasattr(other, "identifier") 
            or other.identifier != self.identifier):
            raise ConversionError("Cannot add different currencies "
                                  "or non-currencies together")
        return Money(self.identifier, self.amount + other.amount)

    def __sub__(self, other):
        if (not hasattr(other, "identifier") 
            or other.identifier != self.identifier):
            raise ConversionError("Cannot subtract different currencies "
                                  "or non-currencies together")
        return Money(self.identifier, self.amount - other.amount) 

    def __mul__(self, other):
        if hasattr(other, "identifier"):
            raise ConversionError("Cannot multiply currency "
                                  "with any currency")
        return Money(self.identifier, self.amount * other)

    def __div__(self, other):
        if hasattr(other, "identifier"):
            raise ConversionError("Cannot divide currency "
                                  "with any currency")
        return Money(self.identifier, self.amount / other)

    def __unicode__(self):
        return u"%s %s" % (self.identifier, self.amount)

class Currency(object):
    def to_btc(self, amount):
        raise NotImplementedError
    def from_btc(self, amount):
        raise NotImplementedError

class BTCCurrency(Currency):
    identifier = "BTC"

    def to_btc(self, amount):
        return amount

    def from_btc(self, amount):
        return amount

class BitcoinChartsCurrency(Currency):
    period = "24h"

    def __init__(self):
        self.cache_key = "%s_in_btc" % self.identifier
        self.cache_key_old = "%s_was_in_btc" % self.identifier

    def populate_cache(self):
        try:
            f = urllib2.urlopen(
                u"http://bitcoincharts.com/t/weighted_prices.json")
            result=f.read()
            j=json.loads(result)
            base_price = j[self.identifier]
            cache.set(self.cache_key, base_price, 60*60)
            #print result
        except:
            print "Unexpected error:", sys.exc_info()[0]
        
        if not cache.get(self.cache_key):
            if not cache.get(self.cache_key_old):
                raise TemporaryConversionError(
                    "Cache not enabled, reliable exchange rate is not available for %s" % self.identifier)
            cache.set(self.cache_key, cache.get(self.cache_key_old), 60*60)

        cache.set(self.cache_key_old, cache.get(self.cache_key), 60*60*24*7)

    def get_factor(self):
        cached = cache.get(self.cache_key)
        if cached:
            factor = cached[self.period]
        else:
            self.populate_cache()
            factor = cache.get(self.cache_key)[self.period]
        return decimal.Decimal(factor)

    def to_btc(self, amount):
        return amount * self.get_factor()

    def from_btc(self, amount):
        return amount / self.get_factor()

class EURCurrency(BitcoinChartsCurrency):
    identifier = "EUR"

class USDCurrency(BitcoinChartsCurrency):
    identifier = "USD"

exchange = Exchange()

# simple utility functions for conversions

CURRENCY_CHOICES = (
    ('BTC', 'BTC'),
    ('USD', 'USD'),
    ('EUR', 'EUR'),
    ('AUD', 'AUD'),
    ('BRL', 'BRL'),
    ('CAD', 'CAD'),
    ('CHF', 'CHF'),
    ('CNY', 'CNY'),
    ('DKK', 'DKK'),
    ('GBP', 'GBP'),
    ('HKD', 'HKD'),
    ('JPY', 'JPY'),
    ('NZD', 'NZD'),
    ('PLN', 'PLN'),
    ('RUB', 'RUB'),
    ('SEK', 'SEK'),
    ('SLL', 'SLL'),
)

RATE_PERIOD_CHOICES=("24h", "7d", "30d",)

def get_rate_table():
    cache_key="bitcoincharts_all"
    cache_key_old="bitcoincharts_all_old"
    if not cache.get(cache_key):
        try:
            f = urllib2.urlopen(
                u"http://bitcoincharts.com/t/weighted_prices.json")
            result=f.read()
            j=json.loads(result)
            cache.set(cache_key, j, 60*60)
            print result
        except:
            print "Unexpected error:", sys.exc_info()[0]

        if not cache.get(cache_key):
            if not cache.get(cache_key_old):
                raise TemporaryConversionError(
                    "Cache not enabled, reliable exchange rate is not available")
            cache.set(cache_key, cache.get(cache_key_old), 60*60)

        cache.set(cache_key_old, cache.get(cache_key), 60*60*24*7)
    return cache.get(cache_key)

def currency_list():
    return get_rate_table().keys()

def get_currency_rate(currency="USD", rate_period="24h"):
    try:
        return Decimal(get_rate_table()[currency][rate_period])
    except KeyError:
        return None

def btc2currency(amount, currency="USD", rate_period="24h"):
    rate=get_currency_rate(currency, rate_period)
    if rate==None:
        return None
    return (amount*rate).quantize(Decimal("0.01"))

def currency2btc(amount, currency="USD", rate_period="24h"):
    rate=get_currency_rate(currency, rate_period)
    if rate==None:
        return None
    return (amount/rate).quantize(Decimal("0.00000001"))






"""
  Copyright 2011 Jeff Garzik

  AuthServiceProxy has the following improvements over python-jsonrpc's
  ServiceProxy class:

  - HTTP connections persist for the life of the AuthServiceProxy object
    (if server supports HTTP/1.1)
  - sends protocol 'version', per JSON-RPC 1.1
  - sends proper, incrementing 'id'
  - sends Basic HTTP authentication headers
  - parses all JSON numbers that look like floats as Decimal
  - uses standard Python json lib

  Previous copyright, from python-jsonrpc/jsonrpc/proxy.py:

  Copyright (c) 2007 Jan-Klaas Kollhof

  This file is part of jsonrpc.

  jsonrpc is free software; you can redistribute it and/or modify
  it under the terms of the GNU Lesser General Public License as published by
  the Free Software Foundation; either version 2.1 of the License, or
  (at your option) any later version.

  This software is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU Lesser General Public License for more details.

  You should have received a copy of the GNU Lesser General Public License
  along with this software; if not, write to the Free Software
  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
"""

import httplib
import base64
import json
import decimal
import urlparse

USER_AGENT = "AuthServiceProxy/0.1"

HTTP_TIMEOUT = 30

class JSONRPCException(Exception):
    def __init__(self, rpcError):
        Exception.__init__(self)
        self.error = rpcError

class AuthServiceProxy(object):
    def __init__(self, serviceURL, serviceName=None):
        self.__serviceURL = serviceURL
        self.__serviceName = serviceName
        self.__url = urlparse.urlparse(serviceURL)
        if self.__url.port is None:
            port = 80
        else:
            port = self.__url.port
        self.__idcnt = 0
        authpair = "%s:%s" % (self.__url.username, self.__url.password)
        self.__authhdr = "Basic %s" % (base64.b64encode(authpair))
        if self.__url.scheme == 'https':
            self.__conn = httplib.HTTPSConnection(self.__url.hostname, port, None, None,False,
                    HTTP_TIMEOUT)
        else:
            self.__conn = httplib.HTTPConnection(self.__url.hostname, port, False,
                    HTTP_TIMEOUT)

    def __getattr__(self, name):
        if self.__serviceName != None:
            name = "%s.%s" % (self.__serviceName, name)
        return AuthServiceProxy(self.__serviceURL, name)

    def __call__(self, *args):
        self.__idcnt += 1

        postdata = json.dumps({
             'version': '1.1',
             'method': self.__serviceName,
             'params': args,
             'id': self.__idcnt})
        self.__conn.request('POST', self.__url.path, postdata,
                 { 'Host' : self.__url.hostname,
                     'User-Agent' : USER_AGENT,
                     'Authorization' : self.__authhdr,
                     'Content-type' : 'application/json' })

        httpresp = self.__conn.getresponse()
        if httpresp is None:
            raise JSONRPCException({
              'code' : -342, 'message' : 'missing HTTP response from server'})

        resp = json.loads(httpresp.read(), parse_float=decimal.Decimal)
        #print resp
        if resp['error'] != None:
            raise JSONRPCException(unicode(resp['error']))
        elif 'result' not in resp:
            raise JSONRPCException({
                'code' : -343, 'message' : 'missing JSON-RPC result'})
        else:
            return resp['result']





# vim: tabstop=4 expandtab autoindent shiftwidth=4 fileencoding=utf-8

import decimal

import mock

from django_bitcoin import utils

## bitcoin mock objects

## Patch your test cases which access bitcoin with these decorators
## eg:
#@mock.patch('django_bitcoin.utils.bitcoind', new=mock_bitcoin_objects.mock_bitcoind)
#@mock.patch('django_bitcoin.models.bitcoind', new=mock_bitcoin_objects.mock_bitcoind)
#def test_wallet_received():
#    ...

## FIRST
mock_bitcoind = mock.Mock(wraps=utils.bitcoind, spec=utils.bitcoind)

mock_received_123 = mock.Mock()
mock_received_123.return_value = decimal.Decimal(123)

mock_bitcoind.total_received = mock.mocksignature(utils.bitcoind.total_received, mock=mock_received_123)

mock_bitcoind.send = mock.mocksignature(utils.bitcoind.send)

mock_bitcoind_address = mock.Mock()
mock_bitcoind_address.return_value = '15EJtRsZAwwxtC4AiUS1ZsypGzk8WDLRFn'

mock_bitcoind.create_address = mock.mocksignature(utils.bitcoind.create_address, mock=mock_bitcoind_address)

## SECOND
mock_bitcoind_other = mock.Mock(wraps=utils.bitcoind, spec=utils.bitcoind)

mock_received_65535 = mock.Mock()
mock_received_65535.return_value = decimal.Decimal(65535)

mock_bitcoind_other.total_received = mock.mocksignature(utils.bitcoind.total_received, mock=mock_received_65535)

mock_bitcoind_other.send = mock.mocksignature(utils.bitcoind.send)

mock_bitcoind_other_address = mock.Mock()
mock_bitcoind_other_address.return_value = '15EJtRsZAwwxtC4AiUS1ZsypGzk8WDLRFn'

mock_bitcoind_other.create_address = mock.mocksignature(utils.bitcoind.create_address, mock=mock_bitcoind_other_address)

# EOF




from distutils.core import setup

setup(name='django-bitcoin',
      version='0.1',
      description='bitcoin payment management for django',
      author='Jeremias Kangas',
      url='https://github.com/kangasbros/django-bitcoin',
      requires=["qrcode (>2.3.1)", "South (>0.7.4)"],
      packages=['django_bitcoin', 
                'django_bitcoin.management',
                'django_bitcoin.management.commands',
                'django_bitcoin.jsonrpc'],
     )




from __future__ import with_statement

import datetime
import random
import hashlib
import base64
from decimal import Decimal

from django.db import models
from django.contrib.sites.models import Site
from django.contrib.auth.models import User

from django_bitcoin.utils import *
from django_bitcoin.utils import bitcoind
from django_bitcoin import settings

from django.utils.translation import ugettext as _

currencies=(
    (1, "USD"), 
    (2, "EUR"), 
    (3, "BTC")
)

confirmation_choices=(
    (0, "0, (quick, recommended)"), 
    (1, "1, (safer, slower for the buyer)"), 
    (5, "5, (for the paranoid, not recommended)")
)


class Transaction(models.Model):
    created_at = models.DateTimeField(default=datetime.datetime.now)
    amount = models.DecimalField(
        max_digits=16, 
        decimal_places=8, 
        default=Decimal("0.0"))
    address = models.CharField(max_length=50)

class BitcoinAddress(models.Model):
    address = models.CharField(max_length=50, unique=True)
    created_at = models.DateTimeField(default=datetime.datetime.now)
    active = models.BooleanField(default=False)
    least_received = models.DecimalField(max_digits=16, decimal_places=8, default=Decimal(0))
    label = models.CharField(max_length=50, blank=True, null=True, default=None)

    class Meta:
        verbose_name_plural = 'Bitcoin addresses'

    def received(self, minconf=settings.BITCOIN_MINIMUM_CONFIRMATIONS):
        r=bitcoind.total_received(self.address, minconf=minconf)
        if r>self.least_received:
            self.least_received=r
            self.save()
        return r

    def __unicode__(self):
        if self.label:
            return u'%s (%s)' % (self.label, self.address)
        return self.address

@transaction.commit_on_success
def new_bitcoin_address():
    bp=BitcoinAddress.objects.filter(active=False)
    if len(bp)<1:
        refill_payment_queue()
        bp=BitcoinAddress.objects.filter(active=False)
    bp=bp[0]
    bp.active=True
    bp.save()
    return bp

class Payment(models.Model):
    description = models.CharField(
        max_length=255, 
        blank=True)
    address = models.CharField(
        max_length=50)
    amount = models.DecimalField(
        max_digits=16, 
        decimal_places=8, 
        default=Decimal("0.0"))
    amount_paid = models.DecimalField(
        max_digits=16, 
        decimal_places=8, 
        default=Decimal("0.0"))
    active = models.BooleanField(default=False)

    created_at = models.DateTimeField(default=datetime.datetime.now)
    updated_at = models.DateTimeField()

    paid_at = models.DateTimeField(null=True, default=None)

    withdrawn_total = models.DecimalField(
        max_digits=16, 
        decimal_places=8, 
        default=Decimal("0.0"))

    transactions = models.ManyToManyField(Transaction)
    
    def calculate_amount(self, proportion):
        return quantitize_bitcoin(
            Decimal((proportion/Decimal("100.0"))*self.amount))

    def add_transaction(self, amount, address):
        self.withdrawn_total += amount
        bctrans = self.transactions.create(
            amount=amount,
            address=address)
        self.save()

        return bctrans
    
    def withdraw_proportion(self, address, proportion):
        if proportion<=Decimal("0") or proportion>Decimal("100"):
            raise Exception("Illegal proportion.")

        amount = self.calculate_amount(proportion)

        if self.amount-self.withdrawn_total > amount:
            raise Exception("Trying to withdraw too much.")

        self.add_transaction(amount, address)
        bitcoind.send(address, amount)

    @classmethod
    def withdraw_proportion_all(cls, address, bitcoin_payments_proportions):
        """hash BitcoinPayment -> Proportion"""
        final_amount=Decimal("0.0")
        print bitcoin_payments_proportions
        for bp, proportion in bitcoin_payments_proportions.iteritems():
            am=bp.calculate_amount(proportion)
            final_amount+=am
            bp.add_transaction(am, address)
        bitcoind.send(address, final_amount)
        return True        

    def withdraw_amounts(self, addresses_shares):
        """hash address -> percentage (string -> Decimal)"""
        if self.amount_paid<self.amount:
            raise Exception("Not paid.")
        if self.withdrawn_at:
            raise Exception("Trying to withdraw again.")
        if sum(addresses_shares.values())>100:
            raise Exception("Sum of proportions must be <=100.")
        #self.withdraw_addresses=",".join(addresses)
        #self.withdraw_proportions=",".join([str(x) for x in proportions])
        amounts=[]
        for p in addresses_shares.values():
            if p<=0:
                raise Exception()
            am=quantitize_bitcoin(Decimal((p/Decimal("100.0"))*self.amount))
            amounts.append(am)
        #self.withdraw_proportions=",".join([str(x) for x in ])
        if sum(amounts)>self.amount:
            raise Exception("Sum of calculated amounts exceeds funds.")
        return amounts
    
    @classmethod
    def calculate_amounts(cls, bitcoinpayments, addresses_shares):
        amounts_all=[Decimal("0.0") for _i in addresses_shares]
        for amount, payment in zip(amounts_all, bitcoinpayments):
            withdrawn=payment.withdraw_amounts(addresses_shares)
            amounts_all=[(w+total) for w, total in zip(withdrawn, amounts_all)]
        return amounts_all

    @classmethod
    def withdraw_all(cls, bitcoinpayments, addresses_shares):
        #if len(bitcoinpayments)!=len(addresses_shares):
        #    raise Exception("")
        amounts_all=Payment.calculate_amounts(bitcoinpayments, addresses_shares)
        for bp in bitcoinpayments:
            am=bp.withdraw_amounts(addresses_shares)
            bp.withdraw_addresses=",".join(addresses_shares.keys())
            bp.withdraw_proportions=",".join(
                [str(x) for x in addresses_shares.values()])
            bp.withdraw_amounts=",".join(
                [str(x) for x in am])
            bp.withdrawn_at=datetime.datetime.now()
            bp.withdrawn_total=sum(am)
            bp.save()
        for i, share in enumerate(addresses_shares.keys()):
            bitcoind.send(share, amounts_all[i])
        return True

    def is_paid(self, minconf=1):
        if self.paid_at:
            return True
        self.update_payment(minconf=minconf)
        return self.amount_paid>=self.amount

    def getbalance(self, minconf=1):
        return bitcoind.total_received(self.address, minconf=minconf)

    def update_payment(self, minconf=1):
        new_amount=Decimal(bitcoin_getbalance(self.address, minconf=minconf))
        print "blaa", new_amount, self.address
        if new_amount>=self.amount:
            self.amount_paid=new_amount
            self.paid_at=datetime.datetime.now()
            self.save()
        #elif (datetime.datetime.now()-self.updated_at)>datetime.timedelta(hours=PAYMENT_VALID_HOURS):
        #    self.deactivate()

    def deactivate(self):
        return False
        if self.amount_paid > Decimal("0"):
            return False
        self.active=False
        self.description=""
        self.save()
        return True
    
    def save(self, **kwargs):
        self.updated_at = datetime.datetime.now()
        return super(Payment, self).save(**kwargs)

    def __unicode__(self):
        return unicode(self.amount_paid)

    @models.permalink
    def get_absolute_url(self):
        return ('view_or_url_name',)

class WalletTransaction(models.Model):
    created_at = models.DateTimeField(default=datetime.datetime.now)
    from_wallet = models.ForeignKey(
        'Wallet', 
        related_name="sent_transactions")
    to_wallet = models.ForeignKey(
        'Wallet', 
        null=True, 
        related_name="received_transactions")
    to_bitcoinaddress = models.CharField(
        max_length=50, 
        blank=True)
    amount = models.DecimalField(
        max_digits=16, 
        decimal_places=8, 
        default=Decimal("0.0"))
    description = models.CharField(max_length=100, blank=True)
    
    def __unicode__(self):
        if self.from_wallet and self.to_wallet:
            return u"Wallet transaction "+unicode(self.amount)
        elif self.from_wallet and self.to_bitcoinaddress:
            return u"Outgoing bitcoin transaction "+unicode(self.amount)
        return u"Fee "+unicode(self.amount)

class Wallet(models.Model):
    created_at = models.DateTimeField(default=datetime.datetime.now)
    updated_at = models.DateTimeField()

    label = models.CharField(max_length=50, blank=True)
    addresses = models.ManyToManyField(BitcoinAddress)
    transactions_with = models.ManyToManyField(
        'self',
        through=WalletTransaction,
        symmetrical=False)

    def __unicode__(self):
        return u"%s: %s" % (self.label,
                            self.created_at.strftime('%Y-%m-%d %H:%M'))

    def save(self):
        '''Assings a wallet label if the wallet doesn't have one.'''
        super(Wallet, self).save(*args, **kwargs)
        if not self.label:
            self.label = 'Wallet #%d' % self.id
            super(Wallet, self).save(*args, **kwargs) 

    def receiving_address(self, fresh_addr=True):
        usable_addresses = self.addresses.filter(active=True).order_by("id")
        if fresh_addr:
            usable_addresses = usable_addresses.filter(least_received=Decimal(0))
        if usable_addresses.count():
            return usable_addresses[0].address
        addr=new_bitcoin_address()
        self.addresses.add(addr)
        return addr.address

    def static_receiving_address(self):
        '''
        Returns a static receiving address for this Wallet object.
        '''
        return self.receiving_address(fresh_addr=False)

    def send_to_wallet(self, otherWallet, amount, description=''):
        if amount>self.total_balance():
            raise Exception(_("Trying to send too much"))
        if self==otherWallet:
            raise Exception(_("Can't send to self-wallet"))
        if not otherWallet.id or not self.id:
            raise Exception(_("Some of the wallets not saved"))
        return WalletTransaction.objects.create(
            amount=amount,
            from_wallet=self,
            to_wallet=otherWallet,
            description=description)
    
    def send_to_address(self, address, amount, description=''):
        if Decimal(amount)<Decimal(0):
            raise Exception(_("Trying to send too much"))
        if Decimal(amount)>self.total_balance():
            raise Exception(_("Trying to send too much"))
        bwt = WalletTransaction.objects.create(
            amount=amount,
            from_wallet=self,
            to_bitcoinaddress=address,
            description=description)
        try:
            result=bitcoind.send(address, amount)
        except JSONRPCException:
            bwt.delete()
            raise
        # check if a transaction fee exists, and deduct it from the wallet
        # TODO: because fee can't be known beforehand, can result in negative wallet balance.
        # currently isn't much of a issue, but might be in the future, depending of the application
        transaction=bitcoind.gettransaction(result)
        fee_transaction=None
        if Decimal(transaction['fee'])<Decimal(0):
            fee_transaction = WalletTransaction.objects.create(
                amount=Decimal(transaction['fee'])*Decimal(-1),
                from_wallet=self)
        return (bwt, fee_transaction)

    def total_received(self):
        """docstring for getreceived"""
        s=sum([a.received() for a in self.addresses.all()])
        rt=self.received_transactions.aggregate(models.Sum("amount"))['amount__sum'] or Decimal(0)
        return (s+rt)

    def total_sent(self):
        return self.sent_transactions.aggregate(models.Sum("amount"))['amount__sum'] or Decimal(0)

    def total_balance(self):
        return self.total_received() - self.total_sent()

    def total_received_unconfirmed(self):
        """docstring for getreceived"""
        s=sum([a.received(minconf=0) for a in self.addresses.all()])
        rt=self.received_transactions.aggregate(models.Sum("amount"))['amount__sum'] or Decimal(0)
        return (s+rt)

    def total_balance_unconfirmed(self):
        return self.total_received_unconfirmed() - self.total_sent()

    def save(self, **kwargs):
        self.updated_at = datetime.datetime.now()
        super(Wallet, self).save(**kwargs)

### Maybe in the future

# class FiatWalletTransaction(models.Model):
#     """Transaction for storing fiat currencies"""
#     pass

# class FiatWallet(models.Model):
#     """Wallet for storing fiat currencies"""
#     pass

# class BitcoinEscrow(models.Model):
#     """Bitcoin escrow payment"""
    
#     created_at = models.DateTimeField(auto_now_add=True)
#     updated_at = models.DateTimeField(auto_now=True)

#     seller = models.ForeignKey(User)
    
#     bitcoin_payment = models.ForeignKey(Payment)

#     confirm_hash = models.CharField(max_length=50, blank=True)
    
#     buyer_address = models.TextField()
#     buyer_phone = models.CharField(max_length=20, blank=True)
#     buyer_email = models.EmailField(max_length=75)
    
#     def save(self, **kwargs):
#         super(BitcoinEscrow, self).save(**kwargs)
#         if not self.confirm_hash:
#             self.confirm_hash=generateuniquehash(
#                 length=32, 
#                 extradata=str(self.id))
#             super(BitcoinEscrow, self).save(**kwargs)
    
#     @models.permalink
#     def get_absolute_url(self):
#         return ('view_or_url_name',)

def refill_payment_queue():
    c=Payment.objects.filter(active=False).count()
    if settings.BITCOIN_PAYMENT_BUFFER_SIZE>c:
        for i in range(0,settings.BITCOIN_PAYMENT_BUFFER_SIZE-c):
            bp=Payment()
            bp.address=bitcoind.create_address()
            bp.save()
    c=BitcoinAddress.objects.filter(active=False).count()
    if settings.BITCOIN_ADDRESS_BUFFER_SIZE>c:
        for i in range(0,settings.BITCOIN_ADDRESS_BUFFER_SIZE-c):
            ba=BitcoinAddress()
            ba.address=bitcoind.create_address()
            ba.save()

def update_payments():
    if not cache.get('last_full_check'):
        cache.set('bitcoinprice', cache.get('bitcoinprice_old'))
    bps=BitcoinPayment.objects.filter(active=True)
    for bp in bps:
        bp.amount_paid=Decimal(bitcoin_getbalance(bp.address))
        bp.save()
        print bp.amount
        print bp.amount_paid

@transaction.commit_on_success
def new_bitcoin_payment(amount):
    bp=BitcoinPayment.objects.filter(active=False)
    if len(bp)<1:
        refill_payment_queue()
        bp=BitcoinPayment.objects.filter(active=False)
    bp=bp[0]
    bp.active=True
    bp.amount=amount
    bp.save()
    return bp

def getNewBitcoinPayment(amount):
    warnings.warn("Use new_bitcoin_payment(amount) instead",
                  DeprecationWarning)
    return new_bitcoin_payment(amount)

@transaction.commit_on_success
def new_bitcoin_payment_eur(amount):
    print bitcoinprice_eur()
    return new_bitcoin_payment(Decimal(amount)/Decimal(bitcoinprice_eur()['24h']))

def getNewBitcoinPayment_eur(amount):
    return new_bitcoin_payment_eur(amount)

# initialize the conversion module

from django_bitcoin import currency

from django.core import urlresolvers
from django.utils import importlib

for dottedpath in settings.BITCOIN_CURRENCIES:
    mod, func = urlresolvers.get_mod_func(dottedpath)
    klass = getattr(importlib.import_module(mod), func)
    currency.exchange.register_currency(klass())

# EOF


